/*

==============================头像自动分割（抠头像）接口 人体头发抠图接口==============================
*/


package com.WxPersonApi;

import com.alibaba.fastjson.JSON; // 依赖包下载地址 https://mvnrepository.com/

import javax.imageio.ImageIO;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class Hair {
    private final String HairAPIServer = "https://person.market.alicloudapi.com/segment";
    private final String HairAPIUri = "/person/hair";
    private final int MaxPhotoEdge = 2000;


    private String base64(byte[] bytes) {
        return Base64.getEncoder().encodeToString(bytes);
    }

    public byte[] resizeAndReadImage(File srcFile) throws IOException {
        ByteArrayOutputStream out = null;
        try {
            // 构造Image对象
            BufferedImage src = ImageIO.read(srcFile);
            int width = src.getWidth();
            int height = src.getHeight();

            int maxEdge = Math.max(width, height);

            // 接口要求图片最长边不超过1500个像素，超过的话，需要先做缩放
            if (maxEdge <= this.MaxPhotoEdge) {
                FileInputStream inputFile = new FileInputStream(srcFile);
                byte[] buffer = new byte[(int) srcFile.length()];
                inputFile.read(buffer);
                inputFile.close();
                return buffer;
            }
            float ratio = (float) this.MaxPhotoEdge / (float) maxEdge;
            int newWidth = (int) ((float) width * ratio);
            int newHeight = (int) ((float) height * ratio);

            BufferedImage tag = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_INT_RGB);
            tag.getGraphics().drawImage(src, 0, 0, newWidth, newHeight, null);

            out = new ByteArrayOutputStream();
            ImageIO.write(tag, "JPEG", out);
            return out.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (out != null) {
                out.close();
            }
        }
        return null;
    }


    private String httpPost(String httpUrl, Map<String, String> headers, String postData) {

        HttpURLConnection connection = null;
        InputStream is = null;
        OutputStream os = null;
        BufferedReader br = null;
        String result = null;
        try {
            URL url = new URL(httpUrl);

            connection = (HttpURLConnection) url.openConnection();
            if (url.getProtocol().toLowerCase().equals("https")) {
                HttpsURLConnection https = (HttpsURLConnection) connection;
                https.setHostnameVerifier(DO_NOT_VERIFY);
                //con = https;
            }

            connection.setRequestMethod("POST");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(60000);

            connection.setDoOutput(true);

            if (headers != null) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    connection.setRequestProperty(entry.getKey(), entry.getValue());
                }
            }

            os = connection.getOutputStream();
            os.write(postData.getBytes());

            if (connection.getResponseCode() == 200) {
                is = connection.getInputStream();
                br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                StringBuffer sbf = new StringBuffer();
                String temp = null;
                while ((temp = br.readLine()) != null) {
                    sbf.append(temp);
                    sbf.append("\r\n");
                }
                result = sbf.toString();

            } else {
                System.out.println("bad status code:" + connection.getResponseCode());
                Map<String, List<String>> map = connection.getHeaderFields();
                for (Map.Entry<String, List<String>> entry : map.entrySet()) {
                    System.out.println("Key : " + entry.getKey() +
                            " ,Value : " + entry.getValue());
                }
                is = connection.getErrorStream();
                if (is != null) {
                    br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                    StringBuffer sbf = new StringBuffer();
                    String temp = null;
                    while ((temp = br.readLine()) != null) {
                        sbf.append(temp);
                        sbf.append("\r\n");
                    }
                    result = sbf.toString();
                }

            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (null != br) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != os) {
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            // 断开与远程地址url的连接
            connection.disconnect();
        }
        return result;
    }


    final static HostnameVerifier DO_NOT_VERIFY = new HostnameVerifier() {

        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };


    private String extractValueFromJsonString(String jsonString, String key) {
        String regex = "\"" + key + "\":([^(,|\\}|\\])]*)";
        Matcher matcher = Pattern.compile(regex).matcher(jsonString);
        while (matcher.find()) {
            String val = matcher.group(1);
            val = val.trim();
            if (val.startsWith("\"")) {
                val = val.substring(1);
            }
            if (val.endsWith("\"")) {
                val = val.substring(0, val.length() - 1);
            }
            return val;
        }
        return null;

    }

    private String apiCall(String apiServer, String apiUri, File imageFile,String Appcode) {
        try {

            byte[] buffer = resizeAndReadImage(imageFile);
            if (buffer == null) {
                return null;
            }
            // BASE64Encoder b64Encoder = new BASE64Encoder();
            String imageBase64 = base64(buffer);
            String fileName = imageFile.getName();
            String fileExt = fileName.substring(fileName.lastIndexOf(".") + 1);
            String photoType = fileExt.replace(".", "");



            Map bodyJson = new HashMap();
            bodyJson.put("photo", imageBase64);
            bodyJson.put("photoType", photoType);


            String body = JSON.toJSONString(bodyJson);


            String accept = "application/json";
            String contentType = "application/octet-stream; charset=utf-8";

            Map<String, String> headers = new HashMap<String, String>();
            headers.put("Accept", accept);
            headers.put("Authorization", "APPCODE "+Appcode);
            headers.put("Content-Type", contentType);


            String result = httpPost(apiServer + apiUri, headers, body);
            if (result == null) {
                System.out.println("httpPost call fail");
                return null;
            }

            System.out.println("result:" + result);
            return result;


        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }

        return null;
    }

    public Boolean Hair(File imageFile, String Appcode) {
        System.out.println("头像自动分割（抠头像）接口 人体头发抠图接口 ...");
        try {

            String result = apiCall(this.HairAPIServer,
                    this.HairAPIUri,
                    imageFile,
                    Appcode);
            if (result == null) {
                return false;
            }


            // 正确做法是使用json解析库，
            // 这里为了减少依赖，直接使用正则表达式解析
            String status = extractValueFromJsonString(result, "status");
            if (!status.equals("0")) {
                System.out.println("bad status:" + status);
                return false;
            }


            return true;
        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }
        return false;
    }


    public static void main(String[] args) {

        /*
         说明：
             此代码是一份调用接口调用示例，
             WXAIPhotoAPI也只是一份调用示例的class，
             并不是接口本身，
             请不要把WXAIPhotoAPI的参数，当成接口本身的参数
        */
        Hair api_test = new Hair();

        String imageFileName = "1.jpg";
        String Appcode = "你的appcode";
        File imageFile = new File(imageFileName);



        api_test.Hair(imageFile,Appcode);

        return;
    }

}